@@ -2,20 +2,23 @@
 # Copyright (c) 2015 raylu <lurayl@gmail.com>
 # Copyright (c) 2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 """Astroid hooks for dateutil"""
 
 import textwrap
 
-from astroid import MANAGER, register_module_extender
+from astroid.brain.helpers import register_module_extender
 from astroid.builder import AstroidBuilder
+from astroid.manager import AstroidManager
 
 
 def dateutil_transform():
-    return AstroidBuilder(MANAGER).string_build(
+    return AstroidBuilder(AstroidManager()).string_build(
         textwrap.dedent(
             """
     import datetime
@@ -26,4 +29,4 @@ def dateutil_transform():
     )
 
 
-register_module_extender(MANAGER, "dateutil.parser", dateutil_transform)
+register_module_extender(AstroidManager(), "dateutil.parser", dateutil_transform)
