@@ -7,24 +7,22 @@
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 Peter Kolbus <peter.kolbus@gmail.com>
 # Copyright (c) 2020 Raphael Gaschignard <raphael@rtpg.co>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 DudeNr33 <3929834+DudeNr33@users.noreply.github.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 import abc
 import collections
-import distutils
 import enum
+import importlib.machinery
 import os
 import sys
 import zipimport
-
-import importlib.machinery
-
-try:
-    from functools import lru_cache
-except ImportError:
-    from backports.functools_lru_cache import lru_cache
+from functools import lru_cache
 
 from . import util
 
@@ -106,7 +104,7 @@ class ImportlibFinder(Finder):
 
     def find_module(self, modname, module_parts, processed, submodule_path):
         if not isinstance(modname, str):
-            raise TypeError("'modname' must be a str, not {}".format(type(modname)))
+            raise TypeError(f"'modname' must be a str, not {type(modname)}")
         if submodule_path is not None:
             submodule_path = list(submodule_path)
         else:
@@ -131,7 +129,7 @@ class ImportlibFinder(Finder):
 
         for entry in submodule_path:
             package_directory = os.path.join(entry, modname)
-            for suffix in [".py", importlib.machinery.BYTECODE_SUFFIXES[0]]:
+            for suffix in (".py", importlib.machinery.BYTECODE_SUFFIXES[0]):
                 package_file_name = "__init__" + suffix
                 file_path = os.path.join(package_directory, package_file_name)
                 if os.path.isfile(file_path):
@@ -162,12 +160,6 @@ class ImportlibFinder(Finder):
                 for p in sys.path
                 if os.path.isdir(os.path.join(p, *processed))
             ]
-        # We already import distutils elsewhere in astroid,
-        # so if it is the same module, we can use it directly.
-        elif spec.name == "distutils" and spec.location in distutils.__path__:
-            # distutils is patched inside virtualenvs to pick up submodules
-            # from the original Python, not from the virtualenv itself.
-            path = list(distutils.__path__)
         else:
             path = [spec.location]
         return path
@@ -225,7 +217,7 @@ class PathSpecFinder(Finder):
             # origin can be either a string on older Python versions
             # or None in case it is a namespace package:
             # https://github.com/python/cpython/pull/5481
-            is_namespace_pkg = spec.origin in ("namespace", None)
+            is_namespace_pkg = spec.origin in {"namespace", None}
             location = spec.origin if not is_namespace_pkg else None
             module_type = ModuleType.PY_NAMESPACE if is_namespace_pkg else None
             spec = ModuleSpec(
@@ -255,7 +247,7 @@ def _is_setuptools_namespace(location):
     try:
         with open(os.path.join(location, "__init__.py"), "rb") as stream:
             data = stream.read(4096)
-    except IOError:
+    except OSError:
         return None
     else:
         extend_path = b"pkgutil" in data and b"extend_path" in data
@@ -274,7 +266,7 @@ def _cached_set_diff(left, right):
 
 def _precache_zipimporters(path=None):
     """
-    For each path that has not been already cached 
+    For each path that has not been already cached
     in the sys.path_importer_cache, create a new zipimporter
     instance and add it into the cache.
     Return a dict associating all paths, stored in the cache, to corresponding
@@ -293,6 +285,7 @@ def _precache_zipimporters(path=None):
     req_paths = tuple(path or sys.path)
     cached_paths = tuple(pic)
     new_paths = _cached_set_diff(req_paths, cached_paths)
+    # pylint: disable=no-member
     for entry_path in new_paths:
         try:
             pic[entry_path] = zipimport.zipimporter(entry_path)
@@ -321,7 +314,7 @@ def _search_zip(modpath, pic):
                     os.path.abspath(filepath) + os.path.sep + os.path.sep.join(modpath),
                     filepath,
                 )
-    raise ImportError("No module named %s" % ".".join(modpath))
+    raise ImportError(f"No module named {'.'.join(modpath)}")
 
 
 def _find_spec_with_path(search_path, modname, module_parts, processed, submodule_path):
@@ -332,7 +325,7 @@ def _find_spec_with_path(search_path, modname, module_parts, processed, submodul
             continue
         return finder, spec
 
-    raise ImportError("No module named %s" % ".".join(module_parts))
+    raise ImportError(f"No module named {'.'.join(module_parts)}")
 
 
 def find_spec(modpath, path=None):
