@@ -7,9 +7,10 @@
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 """
 Data object model, as per https://docs.python.org/3/reference/datamodel.html.
 
@@ -30,21 +31,23 @@ mechanism.
 """
 
 import itertools
-import pprint
 import os
+import pprint
 import types
 from functools import lru_cache
-from typing import Optional
+from typing import TYPE_CHECKING, Optional
 
 import astroid
-from astroid import context as contextmod
-from astroid import exceptions
-from astroid import node_classes
 from astroid import util
+from astroid.context import InferenceContext, copy_context
+from astroid.exceptions import AttributeInferenceError, InferenceError, NoDefault
+from astroid.manager import AstroidManager
+from astroid.nodes import node_classes
 
-# Prevents circular imports
 objects = util.lazy_import("objects")
 
+if TYPE_CHECKING:
+    from astroid.objects import Property
 
 IMPL_PREFIX = "attr_"
 
@@ -122,12 +125,12 @@ class ObjectModel:
 
         if name in self.attributes():
             return getattr(self, IMPL_PREFIX + name)
-        raise exceptions.AttributeInferenceError(target=self._instance, attribute=name)
+        raise AttributeInferenceError(target=self._instance, attribute=name)
 
 
 class ModuleModel(ObjectModel):
     def _builtins(self):
-        builtins_ast_module = astroid.MANAGER.builtins_module
+        builtins_ast_module = AstroidManager().builtins_module
         return builtins_ast_module.special_attributes.lookup("__dict__")
 
     @property
@@ -137,9 +140,7 @@ class ModuleModel(ObjectModel):
     @property
     def attr___path__(self):
         if not self._instance.package:
-            raise exceptions.AttributeInferenceError(
-                target=self._instance, attribute="__path__"
-            )
+            raise AttributeInferenceError(target=self._instance, attribute="__path__")
 
         path_objs = [
             node_classes.Const(
@@ -270,7 +271,7 @@ class FunctionModel(ObjectModel):
             for arg in args.kwonlyargs:
                 try:
                     default = args.default_value(arg.name)
-                except exceptions.NoDefault:
+                except NoDefault:
                     continue
 
                 name = node_classes.Const(arg.name, parent=parent)
@@ -304,17 +305,20 @@ class FunctionModel(ObjectModel):
 
             def infer_call_result(self, caller, context=None):
                 if len(caller.args) > 2 or len(caller.args) < 1:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "Invalid arguments for descriptor binding",
                         target=self,
                         context=context,
                     )
 
-                context = contextmod.copy_context(context)
-                cls = next(caller.args[0].infer(context=context))
+                context = copy_context(context)
+                try:
+                    cls = next(caller.args[0].infer(context=context))
+                except StopIteration as e:
+                    raise InferenceError(context=context, node=caller.args[0]) from e
 
                 if cls is astroid.Uninferable:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "Invalid class inferred", target=self, context=context
                     )
 
@@ -400,6 +404,12 @@ class FunctionModel(ObjectModel):
 
 
 class ClassModel(ObjectModel):
+    def __init__(self):
+        # Add a context so that inferences called from an instance don't recurse endlessly
+        self.context = InferenceContext()
+
+        super().__init__()
+
     @property
     def attr___module__(self):
         return node_classes.Const(self._instance.root().qname())
@@ -419,9 +429,7 @@ class ClassModel(ObjectModel):
     @property
     def attr___mro__(self):
         if not self._instance.newstyle:
-            raise exceptions.AttributeInferenceError(
-                target=self._instance, attribute="__mro__"
-            )
+            raise AttributeInferenceError(target=self._instance, attribute="__mro__")
 
         mro = self._instance.mro()
         obj = node_classes.Tuple(parent=self._instance)
@@ -431,9 +439,7 @@ class ClassModel(ObjectModel):
     @property
     def attr_mro(self):
         if not self._instance.newstyle:
-            raise exceptions.AttributeInferenceError(
-                target=self._instance, attribute="mro"
-            )
+            raise AttributeInferenceError(target=self._instance, attribute="mro")
 
         # pylint: disable=import-outside-toplevel; circular import
         from astroid import bases
@@ -453,7 +459,7 @@ class ClassModel(ObjectModel):
     @property
     def attr___bases__(self):
         obj = node_classes.Tuple()
-        context = contextmod.InferenceContext()
+        context = InferenceContext()
         elts = list(self._instance._inferred_bases(context))
         obj.postinit(elts=elts)
         return obj
@@ -474,10 +480,10 @@ class ClassModel(ObjectModel):
         """
         # pylint: disable=import-outside-toplevel; circular import
         from astroid import bases
-        from astroid import scoped_nodes
+        from astroid.nodes import scoped_nodes
 
         if not self._instance.newstyle:
-            raise exceptions.AttributeInferenceError(
+            raise AttributeInferenceError(
                 target=self._instance, attribute="__subclasses__"
             )
 
@@ -486,7 +492,7 @@ class ClassModel(ObjectModel):
         classes = [
             cls
             for cls in root.nodes_of_class(scoped_nodes.ClassDef)
-            if cls != self._instance and cls.is_subtype_of(qname)
+            if cls != self._instance and cls.is_subtype_of(qname, context=self.context)
         ]
 
         obj = node_classes.List(parent=self._instance)
@@ -558,10 +564,12 @@ class GeneratorModel(FunctionModel):
     def __new__(cls, *args, **kwargs):
         # Append the values from the GeneratorType unto this object.
         ret = super().__new__(cls, *args, **kwargs)
-        generator = astroid.MANAGER.builtins_module["generator"]
+        generator = AstroidManager().builtins_module["generator"]
         for name, values in generator.locals.items():
             method = values[0]
-            patched = lambda cls, meth=method: meth
+
+            def patched(cls, meth=method):
+                return meth
 
             setattr(type(ret), IMPL_PREFIX + name, property(patched))
 
@@ -584,7 +592,7 @@ class AsyncGeneratorModel(GeneratorModel):
     def __new__(cls, *args, **kwargs):
         # Append the values from the AGeneratorType unto this object.
         ret = super().__new__(cls, *args, **kwargs)
-        astroid_builtins = astroid.MANAGER.builtins_module
+        astroid_builtins = AstroidManager().builtins_module
         generator = astroid_builtins.get("async_generator")
         if generator is None:
             # Make it backward compatible.
@@ -592,7 +600,9 @@ class AsyncGeneratorModel(GeneratorModel):
 
         for name, values in generator.locals.items():
             method = values[0]
-            patched = lambda cls, meth=method: meth
+
+            def patched(cls, meth=method):
+                return meth
 
             setattr(type(ret), IMPL_PREFIX + name, property(patched))
 
@@ -630,7 +640,7 @@ class ExceptionInstanceModel(InstanceModel):
 
     @property
     def attr___traceback__(self):
-        builtins_ast_module = astroid.MANAGER.builtins_module
+        builtins_ast_module = AstroidManager().builtins_module
         traceback_type = builtins_ast_module[types.TracebackType.__name__]
         return traceback_type.instantiate_class()
 
@@ -709,7 +719,7 @@ class DictModel(ObjectModel):
             def infer_call_result(self, caller, context=None):
                 yield obj
 
-        meth = next(self._instance._proxied.igetattr(name))
+        meth = next(self._instance._proxied.igetattr(name), None)
         return DictMethodBoundMethod(proxy=meth, bound=self._instance)
 
     @property
@@ -750,8 +760,8 @@ class PropertyModel(ObjectModel):
 
     # pylint: disable=import-outside-toplevel
     def _init_function(self, name):
-        from astroid.node_classes import Arguments
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.node_classes import Arguments
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         args = Arguments()
         args.postinit(
@@ -772,7 +782,7 @@ class PropertyModel(ObjectModel):
 
     @property
     def attr_fget(self):
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         func = self._instance
 
@@ -780,7 +790,7 @@ class PropertyModel(ObjectModel):
             def infer_call_result(self, caller=None, context=None):
                 nonlocal func
                 if caller and len(caller.args) != 1:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "fget() needs a single argument", target=self, context=context
                     )
 
@@ -794,11 +804,11 @@ class PropertyModel(ObjectModel):
 
     @property
     def attr_fset(self):
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         func = self._instance
 
-        def find_setter(func: objects.Property) -> Optional[astroid.FunctionDef]:
+        def find_setter(func: "Property") -> Optional[astroid.FunctionDef]:
             """
             Given a property, find the corresponding setter function and returns it.
 
@@ -815,7 +825,7 @@ class PropertyModel(ObjectModel):
 
         func_setter = find_setter(func)
         if not func_setter:
-            raise exceptions.InferenceError(
+            raise InferenceError(
                 f"Unable to find the setter of property {func.function.name}"
             )
 
@@ -823,7 +833,7 @@ class PropertyModel(ObjectModel):
             def infer_call_result(self, caller=None, context=None):
                 nonlocal func_setter
                 if caller and len(caller.args) != 2:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "fset() needs two arguments", target=self, context=context
                     )
                 yield from func_setter.infer_call_result(caller=caller, context=context)
