@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2009-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Google, Inc.
@@ -14,32 +13,45 @@
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2018 HoverHell <hoverhell@gmail.com>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 Vilnis Termanis <vilnis.termanis@iotics.com>
 # Copyright (c) 2020 Ram Rachum <ram@rachum.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Tushar Sadhwani <86737547+tushar-deepsource@users.noreply.github.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 David Liu <david@cs.toronto.edu>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 doranid <ddandd@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 """this module contains a set of functions to handle python protocols for nodes
 where it makes sense.
 """
 
 import collections
-import operator as operator_mod
-
 import itertools
+import operator as operator_mod
+import sys
+from typing import Any, Generator, List, Optional, Union
 
-from astroid import Store
-from astroid import arguments
-from astroid import bases
-from astroid import context as contextmod
-from astroid import exceptions
-from astroid import decorators
-from astroid import node_classes
-from astroid import helpers
-from astroid import nodes
-from astroid import util
+from astroid import arguments, bases, decorators, helpers, nodes, util
+from astroid.const import Context
+from astroid.context import InferenceContext, copy_context
+from astroid.exceptions import (
+    AstroidIndexError,
+    AstroidTypeError,
+    AttributeInferenceError,
+    InferenceError,
+    NoDefault,
+)
+from astroid.nodes import node_classes
+
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
 
 raw_building = util.lazy_import("raw_building")
 objects = util.lazy_import("objects")
@@ -244,11 +256,7 @@ def _resolve_looppart(parts, assign_path, context):
             index_node = nodes.Const(index)
             try:
                 assigned = stmt.getitem(index_node, context)
-            except (
-                AttributeError,
-                exceptions.AstroidTypeError,
-                exceptions.AstroidIndexError,
-            ):
+            except (AttributeError, AstroidTypeError, AstroidIndexError):
                 continue
             if not assign_path:
                 # we achieved to resolved the assignment path,
@@ -263,12 +271,17 @@ def _resolve_looppart(parts, assign_path, context):
                     yield from _resolve_looppart(
                         assigned.infer(context), assign_path, context
                     )
-                except exceptions.InferenceError:
+                except InferenceError:
                     break
 
 
 @decorators.raise_if_nothing_inferred
-def for_assigned_stmts(self, node=None, context=None, assign_path=None):
+def for_assigned_stmts(
+    self: Union[nodes.For, nodes.Comprehension],
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     if isinstance(self, nodes.AsyncFor) or getattr(self, "is_async", False):
         # Skip inferring of async code for now
         return dict(node=self, unknown=node, assign_path=assign_path, context=context)
@@ -285,13 +298,18 @@ nodes.For.assigned_stmts = for_assigned_stmts
 nodes.Comprehension.assigned_stmts = for_assigned_stmts
 
 
-def sequence_assigned_stmts(self, node=None, context=None, assign_path=None):
+def sequence_assigned_stmts(
+    self: Union[nodes.Tuple, nodes.List],
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     if assign_path is None:
         assign_path = []
     try:
         index = self.elts.index(node)
     except ValueError as exc:
-        raise exceptions.InferenceError(
+        raise InferenceError(
             "Tried to retrieve a node {node!r} which does not exist",
             node=self,
             assign_path=assign_path,
@@ -308,7 +326,12 @@ nodes.Tuple.assigned_stmts = sequence_assigned_stmts
 nodes.List.assigned_stmts = sequence_assigned_stmts
 
 
-def assend_assigned_stmts(self, node=None, context=None, assign_path=None):
+def assend_assigned_stmts(
+    self: Union[nodes.AssignName, nodes.AssignAttr],
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     return self.parent.assigned_stmts(node=self, context=context)
 
 
@@ -344,9 +367,13 @@ def _arguments_infer_argname(self, name, context):
             return
 
     if context and context.callcontext:
-        call_site = arguments.CallSite(context.callcontext, context.extra_context)
-        yield from call_site.infer_argument(self.parent, name, context)
-        return
+        callee = context.callcontext.callee
+        while hasattr(callee, "_proxied"):
+            callee = callee._proxied
+        if getattr(callee, "name", None) == self.parent.name:
+            call_site = arguments.CallSite(context.callcontext, context.extra_context)
+            yield from call_site.infer_argument(self.parent, name, context)
+            return
 
     if name == self.vararg:
         vararg = nodes.const_factory(())
@@ -364,18 +391,33 @@ def _arguments_infer_argname(self, name, context):
     # if there is a default value, yield it. And then yield Uninferable to reflect
     # we can't guess given argument value
     try:
-        context = contextmod.copy_context(context)
+        context = copy_context(context)
         yield from self.default_value(name).infer(context)
         yield util.Uninferable
-    except exceptions.NoDefault:
+    except NoDefault:
         yield util.Uninferable
 
 
-def arguments_assigned_stmts(self, node=None, context=None, assign_path=None):
+def arguments_assigned_stmts(
+    self: nodes.Arguments,
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     if context.callcontext:
+        callee = context.callcontext.callee
+        while hasattr(callee, "_proxied"):
+            callee = callee._proxied
+    else:
+        callee = None
+    if (
+        context.callcontext
+        and node
+        and getattr(callee, "name", None) == node.frame(future=True).name
+    ):
         # reset call context/name
         callcontext = context.callcontext
-        context = contextmod.copy_context(context)
+        context = copy_context(context)
         context.callcontext = None
         args = arguments.CallSite(callcontext, context=context)
         return args.infer_argument(self.parent, node.name, context)
@@ -386,7 +428,12 @@ nodes.Arguments.assigned_stmts = arguments_assigned_stmts
 
 
 @decorators.raise_if_nothing_inferred
-def assign_assigned_stmts(self, node=None, context=None, assign_path=None):
+def assign_assigned_stmts(
+    self: Union[nodes.AugAssign, nodes.Assign, nodes.AnnAssign],
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     if not assign_path:
         yield self.value
         return None
@@ -397,7 +444,12 @@ def assign_assigned_stmts(self, node=None, context=None, assign_path=None):
     return dict(node=self, unknown=node, assign_path=assign_path, context=context)
 
 
-def assign_annassigned_stmts(self, node=None, context=None, assign_path=None):
+def assign_annassigned_stmts(
+    self: nodes.AnnAssign,
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     for inferred in assign_assigned_stmts(self, node, context, assign_path):
         if inferred is None:
             yield util.Uninferable
@@ -427,7 +479,7 @@ def _resolve_assignment_parts(parts, assign_path, context):
             index_node = nodes.Const(index)
             try:
                 assigned = part.getitem(index_node, context)
-            except (exceptions.AstroidTypeError, exceptions.AstroidIndexError):
+            except (AstroidTypeError, AstroidIndexError):
                 return
 
         if not assigned:
@@ -446,12 +498,17 @@ def _resolve_assignment_parts(parts, assign_path, context):
                 yield from _resolve_assignment_parts(
                     assigned.infer(context), assign_path, context
                 )
-            except exceptions.InferenceError:
+            except InferenceError:
                 return
 
 
 @decorators.raise_if_nothing_inferred
-def excepthandler_assigned_stmts(self, node=None, context=None, assign_path=None):
+def excepthandler_assigned_stmts(
+    self: nodes.ExceptHandler,
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     for assigned in node_classes.unpack_infer(self.type):
         if isinstance(assigned, nodes.ClassDef):
             assigned = objects.ExceptionInstance(assigned)
@@ -464,54 +521,50 @@ nodes.ExceptHandler.assigned_stmts = excepthandler_assigned_stmts
 
 
 def _infer_context_manager(self, mgr, context):
-    inferred = next(mgr.infer(context=context))
+    try:
+        inferred = next(mgr.infer(context=context))
+    except StopIteration as e:
+        raise InferenceError(node=mgr) from e
     if isinstance(inferred, bases.Generator):
         # Check if it is decorated with contextlib.contextmanager.
         func = inferred.parent
         if not func.decorators:
-            raise exceptions.InferenceError(
+            raise InferenceError(
                 "No decorators found on inferred generator %s", node=func
             )
 
         for decorator_node in func.decorators.nodes:
-            decorator = next(decorator_node.infer(context=context))
+            decorator = next(decorator_node.infer(context=context), None)
             if isinstance(decorator, nodes.FunctionDef):
                 if decorator.qname() == _CONTEXTLIB_MGR:
                     break
         else:
             # It doesn't interest us.
-            raise exceptions.InferenceError(node=func)
+            raise InferenceError(node=func)
+        try:
+            yield next(inferred.infer_yield_types())
+        except StopIteration as e:
+            raise InferenceError(node=func) from e
 
-        # Get the first yield point. If it has multiple yields,
-        # then a RuntimeError will be raised.
-
-        possible_yield_points = func.nodes_of_class(nodes.Yield)
-        # Ignore yields in nested functions
-        yield_point = next(
-            (node for node in possible_yield_points if node.scope() == func), None
-        )
-        if yield_point:
-            if not yield_point.value:
-                const = nodes.Const(None)
-                const.parent = yield_point
-                const.lineno = yield_point.lineno
-                yield const
-            else:
-                yield from yield_point.value.infer(context=context)
     elif isinstance(inferred, bases.Instance):
         try:
             enter = next(inferred.igetattr("__enter__", context=context))
-        except (exceptions.InferenceError, exceptions.AttributeInferenceError) as exc:
-            raise exceptions.InferenceError(node=inferred) from exc
+        except (InferenceError, AttributeInferenceError, StopIteration) as exc:
+            raise InferenceError(node=inferred) from exc
         if not isinstance(enter, bases.BoundMethod):
-            raise exceptions.InferenceError(node=enter)
+            raise InferenceError(node=enter)
         yield from enter.infer_call_result(self, context)
     else:
-        raise exceptions.InferenceError(node=mgr)
+        raise InferenceError(node=mgr)
 
 
 @decorators.raise_if_nothing_inferred
-def with_assigned_stmts(self, node=None, context=None, assign_path=None):
+def with_assigned_stmts(
+    self: nodes.With,
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     """Infer names and other nodes from a *with* statement.
 
     This enables only inference for name binding in a *with* statement.
@@ -550,7 +603,7 @@ def with_assigned_stmts(self, node=None, context=None, assign_path=None):
             obj = result
             for index in assign_path:
                 if not hasattr(obj, "elts"):
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "Wrong type ({targets!r}) for {node!r} assignment",
                         node=self,
                         targets=node,
@@ -560,7 +613,7 @@ def with_assigned_stmts(self, node=None, context=None, assign_path=None):
                 try:
                     obj = obj.elts[index]
                 except IndexError as exc:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "Tried to infer a nonexistent target with index {index} "
                         "in {node!r}.",
                         node=self,
@@ -569,7 +622,7 @@ def with_assigned_stmts(self, node=None, context=None, assign_path=None):
                         context=context,
                     ) from exc
                 except TypeError as exc:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "Tried to unpack a non-iterable value " "in {node!r}.",
                         node=self,
                         targets=node,
@@ -584,12 +637,17 @@ nodes.With.assigned_stmts = with_assigned_stmts
 
 
 @decorators.raise_if_nothing_inferred
-def named_expr_assigned_stmts(self, node, context=None, assign_path=None):
+def named_expr_assigned_stmts(
+    self: nodes.NamedExpr,
+    node: node_classes.AssignedStmtsPossibleNode,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     """Infer names and other nodes from an assignment expression"""
     if self.target == node:
         yield from self.value.infer(context=context)
     else:
-        raise exceptions.InferenceError(
+        raise InferenceError(
             "Cannot infer NamedExpr node {node!r}",
             node=self,
             assign_path=assign_path,
@@ -601,7 +659,12 @@ nodes.NamedExpr.assigned_stmts = named_expr_assigned_stmts
 
 
 @decorators.yes_if_nothing_inferred
-def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
+def starred_assigned_stmts(
+    self: nodes.Starred,
+    node: node_classes.AssignedStmtsPossibleNode = None,
+    context: Optional[InferenceContext] = None,
+    assign_path: Optional[List[int]] = None,
+) -> Any:
     """
     Arguments:
         self: nodes.Starred
@@ -611,7 +674,7 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
             A list of indices, where each index specifies what item to fetch from
             the inference results.
     """
-    # pylint: disable=too-many-locals,too-many-branches,too-many-statements
+    # pylint: disable=too-many-locals,too-many-statements
     def _determine_starred_iteration_lookups(starred, target, lookups):
         # Determine the lookups for the rhs of the iteration
         itered = target.itered()
@@ -626,9 +689,9 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
                 lookups.append((index, len(element.itered())))
                 _determine_starred_iteration_lookups(starred, element, lookups)
 
-    stmt = self.statement()
+    stmt = self.statement(future=True)
     if not isinstance(stmt, (nodes.Assign, nodes.For)):
-        raise exceptions.InferenceError(
+        raise InferenceError(
             "Statement {stmt!r} enclosing {node!r} " "must be an Assign or For node.",
             node=self,
             stmt=stmt,
@@ -637,14 +700,14 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
         )
 
     if context is None:
-        context = contextmod.InferenceContext()
+        context = InferenceContext()
 
     if isinstance(stmt, nodes.Assign):
         value = stmt.value
         lhs = stmt.targets[0]
 
         if sum(1 for _ in lhs.nodes_of_class(nodes.Starred)) > 1:
-            raise exceptions.InferenceError(
+            raise InferenceError(
                 "Too many starred arguments in the " " assignment targets {lhs!r}.",
                 node=self,
                 targets=lhs,
@@ -654,7 +717,7 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
 
         try:
             rhs = next(value.infer(context))
-        except exceptions.InferenceError:
+        except (InferenceError, StopIteration):
             yield util.Uninferable
             return
         if rhs is util.Uninferable or not hasattr(rhs, "itered"):
@@ -690,7 +753,10 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
 
                 # We're done unpacking.
                 packed = nodes.List(
-                    ctx=Store, parent=self, lineno=lhs.lineno, col_offset=lhs.col_offset
+                    ctx=Context.Store,
+                    parent=self,
+                    lineno=lhs.lineno,
+                    col_offset=lhs.col_offset,
                 )
                 packed.postinit(elts=list(elts))
                 yield packed
@@ -699,7 +765,7 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
     if isinstance(stmt, nodes.For):
         try:
             inferred_iterable = next(stmt.iter.infer(context=context))
-        except exceptions.InferenceError:
+        except (InferenceError, StopIteration):
             yield util.Uninferable
             return
         if inferred_iterable is util.Uninferable or not hasattr(
@@ -716,7 +782,7 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
         target = stmt.target
 
         if not isinstance(target, nodes.Tuple):
-            raise exceptions.InferenceError(
+            raise InferenceError(
                 "Could not make sense of this, the target must be a tuple",
                 context=context,
             )
@@ -724,7 +790,7 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
         lookups = []
         _determine_starred_iteration_lookups(self, target, lookups)
         if not lookups:
-            raise exceptions.InferenceError(
+            raise InferenceError(
                 "Could not make sense of this, needs at least a lookup", context=context
             )
 
@@ -770,7 +836,10 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
                     found_element = element
 
             unpacked = nodes.List(
-                ctx=Store, parent=self, lineno=self.lineno, col_offset=self.col_offset
+                ctx=Context.Store,
+                parent=self,
+                lineno=self.lineno,
+                col_offset=self.col_offset,
             )
             unpacked.postinit(elts=found_element or [])
             yield unpacked
@@ -780,3 +849,58 @@ def starred_assigned_stmts(self, node=None, context=None, assign_path=None):
 
 
 nodes.Starred.assigned_stmts = starred_assigned_stmts
+
+
+@decorators.yes_if_nothing_inferred
+def match_mapping_assigned_stmts(
+    self: nodes.MatchMapping,
+    node: nodes.AssignName,
+    context: Optional[InferenceContext] = None,
+    assign_path: Literal[None] = None,
+) -> Generator[nodes.NodeNG, None, None]:
+    """Return empty generator (return -> raises StopIteration) so inferred value
+    is Uninferable.
+    """
+    return
+    yield
+
+
+nodes.MatchMapping.assigned_stmts = match_mapping_assigned_stmts
+
+
+@decorators.yes_if_nothing_inferred
+def match_star_assigned_stmts(
+    self: nodes.MatchStar,
+    node: nodes.AssignName,
+    context: Optional[InferenceContext] = None,
+    assign_path: Literal[None] = None,
+) -> Generator[nodes.NodeNG, None, None]:
+    """Return empty generator (return -> raises StopIteration) so inferred value
+    is Uninferable.
+    """
+    return
+    yield
+
+
+nodes.MatchStar.assigned_stmts = match_star_assigned_stmts
+
+
+@decorators.yes_if_nothing_inferred
+def match_as_assigned_stmts(
+    self: nodes.MatchAs,
+    node: nodes.AssignName,
+    context: Optional[InferenceContext] = None,
+    assign_path: Literal[None] = None,
+) -> Generator[nodes.NodeNG, None, None]:
+    """Infer MatchAs as the Match subject if it's the only MatchCase pattern
+    else raise StopIteration to yield Uninferable.
+    """
+    if (
+        isinstance(self.parent, nodes.MatchCase)
+        and isinstance(self.parent.parent, nodes.Match)
+        and self.pattern is None
+    ):
+        yield self.parent.parent.subject
+
+
+nodes.MatchAs.assigned_stmts = match_as_assigned_stmts
