@@ -14,10 +14,12 @@
 # Copyright (c) 2020 Becker Awqatty <bawqatty@mide.com>
 # Copyright (c) 2020 Robin Jarry <robin.jarry@6wind.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 """this module contains a set of functions to create astroid trees from scratch
 (build_* functions) or from living object (object_build_* functions)
@@ -29,16 +31,13 @@ import os
 import sys
 import types
 import warnings
+from typing import List, Optional
 
-from astroid import bases
-from astroid import manager
-from astroid import node_classes
-from astroid import nodes
+from astroid import bases, nodes
+from astroid.manager import AstroidManager
+from astroid.nodes import node_classes
 
-
-MANAGER = manager.AstroidManager()
 # the keys of CONST_CLS eg python builtin types
-
 _CONSTANTS = tuple(node_classes.CONST_CLS)
 _BUILTINS = vars(builtins)
 TYPE_NONE = type(None)
@@ -108,7 +107,7 @@ def attach_import_node(node, modname, membername):
     _attach_local_node(node, from_node, membername)
 
 
-def build_module(name, doc=None):
+def build_module(name: str, doc: Optional[str] = None) -> nodes.Module:
     """create and initialize an astroid Module node"""
     node = nodes.Module(name, doc, pure_python=False)
     node.package = False
@@ -120,36 +119,39 @@ def build_class(name, basenames=(), doc=None):
     """create and initialize an astroid ClassDef node"""
     node = nodes.ClassDef(name, doc)
     for base in basenames:
-        basenode = nodes.Name()
-        basenode.name = base
+        basenode = nodes.Name(name=base)
         node.bases.append(basenode)
         basenode.parent = node
     return node
 
 
-def build_function(name, args=None, posonlyargs=None, defaults=None, doc=None):
+def build_function(
+    name,
+    args: Optional[List[str]] = None,
+    posonlyargs: Optional[List[str]] = None,
+    defaults=None,
+    doc=None,
+    kwonlyargs: Optional[List[str]] = None,
+) -> nodes.FunctionDef:
     """create and initialize an astroid FunctionDef node"""
-    args, defaults, posonlyargs = args or [], defaults or [], posonlyargs or []
     # first argument is now a list of decorators
     func = nodes.FunctionDef(name, doc)
-    func.args = argsnode = nodes.Arguments()
-    argsnode.args = []
-    argsnode.posonlyargs = []
-    for arg in args:
-        argsnode.args.append(nodes.Name())
-        argsnode.args[-1].name = arg
-        argsnode.args[-1].parent = argsnode
-    for arg in posonlyargs:
-        argsnode.posonlyargs.append(nodes.Name())
-        argsnode.posonlyargs[-1].name = arg
-        argsnode.posonlyargs[-1].parent = argsnode
-    argsnode.defaults = []
-    for default in defaults:
+    func.args = argsnode = nodes.Arguments(parent=func)
+    argsnode.postinit(
+        args=[nodes.AssignName(name=arg, parent=argsnode) for arg in args or ()],
+        defaults=[],
+        kwonlyargs=[
+            nodes.AssignName(name=arg, parent=argsnode) for arg in kwonlyargs or ()
+        ],
+        kw_defaults=[],
+        annotations=[],
+        posonlyargs=[
+            nodes.AssignName(name=arg, parent=argsnode) for arg in posonlyargs or ()
+        ],
+    )
+    for default in defaults or ():
         argsnode.defaults.append(nodes.const_factory(default))
         argsnode.defaults[-1].parent = argsnode
-    argsnode.kwarg = None
-    argsnode.vararg = None
-    argsnode.parent = func
     if args:
         register_arguments(func)
     return func
@@ -173,7 +175,7 @@ def register_arguments(func, args=None):
         if func.args.kwarg:
             func.set_local(func.args.kwarg, func.args)
     for arg in args:
-        if isinstance(arg, nodes.Name):
+        if isinstance(arg, nodes.AssignName):
             func.set_local(arg.name, arg)
         else:
             register_arguments(func, arg.elts)
@@ -191,6 +193,7 @@ def object_build_function(node, member, localname):
     args = []
     defaults = []
     posonlyargs = []
+    kwonlyargs = []
     for param_name, param in signature.parameters.items():
         if param.kind == inspect.Parameter.POSITIONAL_ONLY:
             posonlyargs.append(param_name)
@@ -200,6 +203,8 @@ def object_build_function(node, member, localname):
             args.append(param_name)
         elif param.kind == inspect.Parameter.VAR_KEYWORD:
             args.append(param_name)
+        elif param.kind == inspect.Parameter.KEYWORD_ONLY:
+            kwonlyargs.append(param_name)
         if param.default is not inspect._empty:
             defaults.append(param.default)
     func = build_function(
@@ -294,11 +299,17 @@ class InspectBuilder:
     FunctionDef and ClassDef nodes and some others as guessed.
     """
 
-    def __init__(self):
+    def __init__(self, manager_instance=None):
+        self._manager = manager_instance or AstroidManager()
         self._done = {}
         self._module = None
 
-    def inspect_build(self, module, modname=None, path=None):
+    def inspect_build(
+        self,
+        module: types.ModuleType,
+        modname: Optional[str] = None,
+        path: Optional[str] = None,
+    ) -> nodes.Module:
         """build astroid from a living module (i.e. using inspect)
         this is used when there is no python source code available (either
         because it's a built-in module or because the .py is not available)
@@ -313,7 +324,7 @@ class InspectBuilder:
             node = build_module(modname)
         node.file = node.path = os.path.abspath(path) if path else path
         node.name = modname
-        MANAGER.cache_module(node)
+        self._manager.cache_module(node)
         node.package = hasattr(module, "__path__")
         self._done = {}
         self.object_build(node, module)
@@ -390,7 +401,7 @@ class InspectBuilder:
         except TypeError:
             modname = None
         if modname is None:
-            if name in ("__new__", "__subclasshook__"):
+            if name in {"__new__", "__subclasshook__"}:
                 # Python 2.5.1 (r251:54863, Sep  1 2010, 22:03:14)
                 # >>> print object.__new__.__module__
                 # None
@@ -414,12 +425,13 @@ class InspectBuilder:
         return False
 
 
-### astroid bootstrapping ######################################################
+# astroid bootstrapping ######################################################
 
 _CONST_PROXY = {}
 
-# TODO : find a nicer way to handle this situation;
+
 def _set_proxied(const):
+    # TODO : find a nicer way to handle this situation;
     return _CONST_PROXY[const.value.__class__]
 
 
@@ -430,7 +442,6 @@ def _astroid_bootstrapping():
     builder = InspectBuilder()
     astroid_builtin = builder.inspect_build(builtins)
 
-    # pylint: disable=redefined-outer-name
     for cls, node_cls in node_classes.CONST_CLS.items():
         if cls is TYPE_NONE:
             proxy = build_class("NoneType")
@@ -459,7 +470,6 @@ def _astroid_bootstrapping():
     builder.object_build(bases.Generator._proxied, types.GeneratorType)
 
     if hasattr(types, "AsyncGeneratorType"):
-        # pylint: disable=no-member; AsyncGeneratorType
         _AsyncGeneratorType = nodes.ClassDef(
             types.AsyncGeneratorType.__name__, types.AsyncGeneratorType.__doc__
         )
