@@ -2,9 +2,11 @@
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 import unittest
 
 try:
@@ -31,13 +33,11 @@ class BrainNumpyCoreFunctionBaseTest(unittest.TestCase):
 
     def _inferred_numpy_func_call(self, func_name, *func_args):
         node = builder.extract_node(
-            """
+            f"""
         import numpy as np
-        func = np.{:s}
-        func({:s})
-        """.format(
-                func_name, ",".join(func_args)
-            )
+        func = np.{func_name:s}
+        func({','.join(func_args):s})
+        """
         )
         return node.infer()
 
