@@ -1,10 +1,13 @@
-# -*- encoding=utf-8 -*-
-# Copyright (c) 2019-2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2019-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2020 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 import unittest
 
 try:
@@ -14,9 +17,7 @@ try:
 except ImportError:
     HAS_NUMPY = False
 
-from astroid import builder
-from astroid import nodes, bases
-from astroid import util
+from astroid import bases, builder, nodes
 
 
 @unittest.skipUnless(HAS_NUMPY, "This test requires the numpy library.")
@@ -109,12 +110,10 @@ class NumpyBrainCoreUmathTest(unittest.TestCase):
 
     def _inferred_numpy_attribute(self, func_name):
         node = builder.extract_node(
-            """
+            f"""
         import numpy.core.umath as tested_module
-        func = tested_module.{:s}
-        func""".format(
-                func_name
-            )
+        func = tested_module.{func_name:s}
+        func"""
         )
         return next(node.infer())
 
@@ -205,13 +204,11 @@ class NumpyBrainCoreUmathTest(unittest.TestCase):
 
     def _inferred_numpy_func_call(self, func_name, *func_args):
         node = builder.extract_node(
-            """
+            f"""
         import numpy as np
-        func = np.{:s}
+        func = np.{func_name:s}
         func()
-        """.format(
-                func_name
-            )
+        """
         )
         return node.infer()
 
@@ -228,14 +225,12 @@ class NumpyBrainCoreUmathTest(unittest.TestCase):
                 self.assertTrue(
                     len(inferred_values) == 1,
                     msg="Too much inferred values ({}) for {:s}".format(
-                        inferred_values, func_
+                        inferred_values[-1].pytype(), func_
                     ),
                 )
                 self.assertTrue(
                     inferred_values[0].pytype() == ".ndarray",
-                    msg="Illicit type for {:s} ({})".format(
-                        func_, inferred_values[-1].pytype()
-                    ),
+                    msg=f"Illicit type for {func_:s} ({inferred_values[-1].pytype()})",
                 )
 
     def test_numpy_core_umath_functions_return_type_tuple(self):
@@ -249,31 +244,23 @@ class NumpyBrainCoreUmathTest(unittest.TestCase):
                 inferred_values = list(self._inferred_numpy_func_call(func_))
                 self.assertTrue(
                     len(inferred_values) == 1,
-                    msg="Too much inferred values ({}) for {:s}".format(
-                        inferred_values, func_
-                    ),
+                    msg=f"Too much inferred values ({inferred_values}) for {func_:s}",
                 )
                 self.assertTrue(
                     inferred_values[-1].pytype() == "builtins.tuple",
-                    msg="Illicit type for {:s} ({})".format(
-                        func_, inferred_values[-1].pytype()
-                    ),
+                    msg=f"Illicit type for {func_:s} ({inferred_values[-1].pytype()})",
                 )
                 self.assertTrue(
                     len(inferred_values[0].elts) == 2,
-                    msg="{} should return a pair of values. That's not the case.".format(
-                        func_
-                    ),
+                    msg=f"{func_} should return a pair of values. That's not the case.",
                 )
                 for array in inferred_values[-1].elts:
                     effective_infer = [m.pytype() for m in array.inferred()]
                     self.assertTrue(
                         ".ndarray" in effective_infer,
                         msg=(
-                            "Each item in the return of {} "
-                            "should be inferred as a ndarray and not as {}".format(
-                                func_, effective_infer
-                            )
+                            f"Each item in the return of {func_} should be inferred"
+                            f" as a ndarray and not as {effective_infer}"
                         ),
                     )
 
