@@ -2,9 +2,10 @@
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 import unittest
 
 try:
@@ -14,8 +15,7 @@ try:
 except ImportError:
     HAS_NUMPY = False
 
-from astroid import builder
-from astroid import nodes
+from astroid import builder, nodes
 
 
 @unittest.skipUnless(HAS_NUMPY, "This test requires the numpy library.")
@@ -24,7 +24,7 @@ class NumpyBrainRandomMtrandTest(unittest.TestCase):
     Test of all the functions of numpy.random.mtrand module.
     """
 
-    #  Map between functions names and arguments names and default values
+    # Map between functions names and arguments names and default values
     all_mtrand = {
         "beta": (["a", "b", "size"], [None]),
         "binomial": (["n", "p", "size"], [None]),
