@@ -13,13 +13,18 @@
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Tushar Sadhwani <86737547+tushar-deepsource@users.noreply.github.com>
+# Copyright (c) 2021 Kian Meng, Ang <kianmeng.ang@gmail.com>
+# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
+# Copyright (c) 2021 pre-commit-ci[bot] <bot@noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 """tests for the astroid builder and rebuilder module"""
 
-import builtins
 import collections
 import os
 import socket
@@ -27,24 +32,26 @@ import sys
 import unittest
 
 import pytest
-from astroid import builder
-from astroid import exceptions
-from astroid import manager
-from astroid import nodes
-from astroid import test_utils
-from astroid import util
-from . import resources
 
-MANAGER = manager.AstroidManager()
-BUILTINS = builtins.__name__
-PY38 = sys.version_info[:2] >= (3, 8)
+from astroid import Instance, builder, nodes, test_utils, util
+from astroid.const import PY38_PLUS
+from astroid.exceptions import (
+    AstroidBuildingError,
+    AstroidSyntaxError,
+    AttributeInferenceError,
+    InferenceError,
+    StatementMissing,
+)
+from astroid.nodes.scoped_nodes import Module
+
+from . import resources
 
 
 class FromToLineNoTest(unittest.TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.astroid = resources.build_file("data/format.py")
 
-    def test_callfunc_lineno(self):
+    def test_callfunc_lineno(self) -> None:
         stmts = self.astroid.body
         # on line 4:
         #    function('aeozrijz\
@@ -66,7 +73,7 @@ class FromToLineNoTest(unittest.TestCase):
         if hasattr(sys, "pypy_version_info"):
             lineno = 4
         else:
-            lineno = 5 if not PY38 else 4
+            lineno = 5 if not PY38_PLUS else 4
         self.assertEqual(strarg.fromlineno, lineno)
         self.assertEqual(strarg.tolineno, lineno)
         namearg = callfunc.args[1]
@@ -95,7 +102,10 @@ class FromToLineNoTest(unittest.TestCase):
             self.assertEqual(arg.fromlineno, 10 + i)
             self.assertEqual(arg.tolineno, 10 + i)
 
-    def test_function_lineno(self):
+    @pytest.mark.skip(
+        "FIXME  http://bugs.python.org/issue10445 (no line number on function args)"
+    )
+    def test_function_lineno(self) -> None:
         stmts = self.astroid.body
         # on line 15:
         #    def definition(a,
@@ -110,12 +120,8 @@ class FromToLineNoTest(unittest.TestCase):
         self.assertIsInstance(return_, nodes.Return)
         self.assertEqual(return_.fromlineno, 18)
         self.assertEqual(return_.tolineno, 18)
-        self.skipTest(
-            "FIXME  http://bugs.python.org/issue10445 "
-            "(no line number on function args)"
-        )
 
-    def test_decorated_function_lineno(self):
+    def test_decorated_function_lineno(self) -> None:
         astroid = builder.parse(
             """
             @decorator
@@ -132,7 +138,7 @@ class FromToLineNoTest(unittest.TestCase):
         self.assertEqual(function.decorators.fromlineno, 2)
         self.assertEqual(function.decorators.tolineno, 2)
 
-    def test_class_lineno(self):
+    def test_class_lineno(self) -> None:
         stmts = self.astroid.body
         # on line 20:
         #    class debile(dict,
@@ -148,7 +154,7 @@ class FromToLineNoTest(unittest.TestCase):
         self.assertEqual(pass_.fromlineno, 22)
         self.assertEqual(pass_.tolineno, 22)
 
-    def test_if_lineno(self):
+    def test_if_lineno(self) -> None:
         stmts = self.astroid.body
         # on line 20:
         #    if aaaa: pass
@@ -163,7 +169,7 @@ class FromToLineNoTest(unittest.TestCase):
         self.assertEqual(if_.orelse[0].fromlineno, 26)
         self.assertEqual(if_.orelse[1].tolineno, 27)
 
-    def test_for_while_lineno(self):
+    def test_for_while_lineno(self) -> None:
         for code in (
             """
             for a in range(4):
@@ -188,7 +194,7 @@ class FromToLineNoTest(unittest.TestCase):
             self.assertEqual(stmt.orelse[0].fromlineno, 6)  # XXX
             self.assertEqual(stmt.orelse[0].tolineno, 6)
 
-    def test_try_except_lineno(self):
+    def test_try_except_lineno(self) -> None:
         astroid = builder.parse(
             """
             try:
@@ -211,7 +217,7 @@ class FromToLineNoTest(unittest.TestCase):
         self.assertEqual(hdlr.tolineno, 5)
         self.assertEqual(hdlr.blockstart_tolineno, 4)
 
-    def test_try_finally_lineno(self):
+    def test_try_finally_lineno(self) -> None:
         astroid = builder.parse(
             """
             try:
@@ -228,7 +234,7 @@ class FromToLineNoTest(unittest.TestCase):
         self.assertEqual(try_.finalbody[0].fromlineno, 5)  # XXX
         self.assertEqual(try_.finalbody[0].tolineno, 5)
 
-    def test_try_finally_25_lineno(self):
+    def test_try_finally_25_lineno(self) -> None:
         astroid = builder.parse(
             """
             try:
@@ -247,7 +253,7 @@ class FromToLineNoTest(unittest.TestCase):
         self.assertEqual(try_.finalbody[0].fromlineno, 7)  # XXX
         self.assertEqual(try_.finalbody[0].tolineno, 7)
 
-    def test_with_lineno(self):
+    def test_with_lineno(self) -> None:
         astroid = builder.parse(
             """
             from __future__ import with_statement
@@ -263,28 +269,29 @@ class FromToLineNoTest(unittest.TestCase):
 
 
 class BuilderTest(unittest.TestCase):
-    def setUp(self):
-        self.builder = builder.AstroidBuilder()
+    def setUp(self) -> None:
+        self.manager = test_utils.brainless_manager()
+        self.builder = builder.AstroidBuilder(self.manager)
 
-    def test_data_build_null_bytes(self):
-        with self.assertRaises(exceptions.AstroidSyntaxError):
+    def test_data_build_null_bytes(self) -> None:
+        with self.assertRaises(AstroidSyntaxError):
             self.builder.string_build("\x00")
 
-    def test_data_build_invalid_x_escape(self):
-        with self.assertRaises(exceptions.AstroidSyntaxError):
+    def test_data_build_invalid_x_escape(self) -> None:
+        with self.assertRaises(AstroidSyntaxError):
             self.builder.string_build('"\\x1"')
 
-    def test_missing_newline(self):
+    def test_missing_newline(self) -> None:
         """check that a file with no trailing new line is parseable"""
         resources.build_file("data/noendingnewline.py")
 
-    def test_missing_file(self):
-        with self.assertRaises(exceptions.AstroidBuildingError):
-            resources.build_file("data/inexistant.py")
+    def test_missing_file(self) -> None:
+        with self.assertRaises(AstroidBuildingError):
+            resources.build_file("data/inexistent.py")
 
-    def test_inspect_build0(self):
+    def test_inspect_build0(self) -> None:
         """test astroid tree build from a living object"""
-        builtin_ast = MANAGER.ast_from_module_name(BUILTINS)
+        builtin_ast = self.manager.ast_from_module_name("builtins")
         # just check type and object are there
         builtin_ast.getattr("type")
         objectastroid = builtin_ast.getattr("object")[0]
@@ -302,16 +309,16 @@ class BuilderTest(unittest.TestCase):
         self.assertIsInstance(builtin_ast["Exception"], nodes.ClassDef)
         self.assertIsInstance(builtin_ast["NotImplementedError"], nodes.ClassDef)
 
-    def test_inspect_build1(self):
-        time_ast = MANAGER.ast_from_module_name("time")
+    def test_inspect_build1(self) -> None:
+        time_ast = self.manager.ast_from_module_name("time")
         self.assertTrue(time_ast)
         self.assertEqual(time_ast["time"].args.defaults, [])
 
-    def test_inspect_build3(self):
+    def test_inspect_build3(self) -> None:
         self.builder.inspect_build(unittest)
 
-    def test_inspect_build_type_object(self):
-        builtin_ast = MANAGER.ast_from_module_name(BUILTINS)
+    def test_inspect_build_type_object(self) -> None:
+        builtin_ast = self.manager.ast_from_module_name("builtins")
 
         inferred = list(builtin_ast.igetattr("object"))
         self.assertEqual(len(inferred), 1)
@@ -325,23 +332,23 @@ class BuilderTest(unittest.TestCase):
         self.assertEqual(inferred.name, "type")
         inferred.as_string()  # no crash test
 
-    def test_inspect_transform_module(self):
+    def test_inspect_transform_module(self) -> None:
         # ensure no cached version of the time module
-        MANAGER._mod_file_cache.pop(("time", None), None)
-        MANAGER.astroid_cache.pop("time", None)
+        self.manager._mod_file_cache.pop(("time", None), None)
+        self.manager.astroid_cache.pop("time", None)
 
-        def transform_time(node):
+        def transform_time(node: Module) -> None:
             if node.name == "time":
                 node.transformed = True
 
-        MANAGER.register_transform(nodes.Module, transform_time)
+        self.manager.register_transform(nodes.Module, transform_time)
         try:
-            time_ast = MANAGER.ast_from_module_name("time")
+            time_ast = self.manager.ast_from_module_name("time")
             self.assertTrue(getattr(time_ast, "transformed", False))
         finally:
-            MANAGER.unregister_transform(nodes.Module, transform_time)
+            self.manager.unregister_transform(nodes.Module, transform_time)
 
-    def test_package_name(self):
+    def test_package_name(self) -> None:
         """test base properties and method of an astroid module"""
         datap = resources.build_file("data/__init__.py", "data")
         self.assertEqual(datap.name, "data")
@@ -353,7 +360,7 @@ class BuilderTest(unittest.TestCase):
         self.assertEqual(datap.name, "data.tmp__init__")
         self.assertEqual(datap.package, 0)
 
-    def test_yield_parent(self):
+    def test_yield_parent(self) -> None:
         """check if we added discard nodes as yield parent (w/ compiler)"""
         code = """
             def yiell(): #@
@@ -369,11 +376,11 @@ class BuilderTest(unittest.TestCase):
         self.assertIsInstance(func.body[1].body[0], nodes.Expr)
         self.assertIsInstance(func.body[1].body[0].value, nodes.Yield)
 
-    def test_object(self):
+    def test_object(self) -> None:
         obj_ast = self.builder.inspect_build(object)
         self.assertIn("__setattr__", obj_ast)
 
-    def test_newstyle_detection(self):
+    def test_newstyle_detection(self) -> None:
         data = """
             class A:
                 "old style"
@@ -403,7 +410,7 @@ class BuilderTest(unittest.TestCase):
         self.assertTrue(mod_ast["D"].newstyle)
         self.assertTrue(mod_ast["F"].newstyle)
 
-    def test_globals(self):
+    def test_globals(self) -> None:
         data = """
             CSTE = 1
 
@@ -420,12 +427,12 @@ class BuilderTest(unittest.TestCase):
         self.assertIsInstance(astroid.getattr("CSTE")[0], nodes.AssignName)
         self.assertEqual(astroid.getattr("CSTE")[0].fromlineno, 2)
         self.assertEqual(astroid.getattr("CSTE")[1].fromlineno, 6)
-        with self.assertRaises(exceptions.AttributeInferenceError):
+        with self.assertRaises(AttributeInferenceError):
             astroid.getattr("CSTE2")
-        with self.assertRaises(exceptions.InferenceError):
+        with self.assertRaises(InferenceError):
             next(astroid["global_no_effect"].ilookup("CSTE2"))
 
-    def test_socket_build(self):
+    def test_socket_build(self) -> None:
         astroid = self.builder.module_build(socket)
         # XXX just check the first one. Actually 3 objects are inferred (look at
         # the socket module) but the last one as those attributes dynamically
@@ -436,7 +443,7 @@ class BuilderTest(unittest.TestCase):
             self.assertIn("close", fclass)
             break
 
-    def test_gen_expr_var_scope(self):
+    def test_gen_expr_var_scope(self) -> None:
         data = "l = list(n for n in range(10))\n"
         astroid = builder.parse(data, __name__)
         # n unavailable outside gen expr scope
@@ -446,15 +453,15 @@ class BuilderTest(unittest.TestCase):
         self.assertIsNot(n.scope(), astroid)
         self.assertEqual([i.__class__ for i in n.infer()], [util.Uninferable.__class__])
 
-    def test_no_future_imports(self):
+    def test_no_future_imports(self) -> None:
         mod = builder.parse("import sys")
         self.assertEqual(set(), mod.future_imports)
 
-    def test_future_imports(self):
+    def test_future_imports(self) -> None:
         mod = builder.parse("from __future__ import print_function")
         self.assertEqual({"print_function"}, mod.future_imports)
 
-    def test_two_future_imports(self):
+    def test_two_future_imports(self) -> None:
         mod = builder.parse(
             """
             from __future__ import print_function
@@ -463,7 +470,7 @@ class BuilderTest(unittest.TestCase):
         )
         self.assertEqual({"print_function", "absolute_import"}, mod.future_imports)
 
-    def test_inferred_build(self):
+    def test_inferred_build(self) -> None:
         code = """
             class A: pass
             A.type = "class"
@@ -479,7 +486,54 @@ class BuilderTest(unittest.TestCase):
         self.assertIn("assign_type", lclass.locals)
         self.assertIn("type", lclass.locals)
 
-    def test_augassign_attr(self):
+    def test_infer_can_assign_regular_object(self) -> None:
+        mod = builder.parse(
+            """
+            class A:
+                pass
+            a = A()
+            a.value = "is set"
+            a.other = "is set"
+        """
+        )
+        obj = list(mod.igetattr("a"))
+        self.assertEqual(len(obj), 1)
+        obj = obj[0]
+        self.assertIsInstance(obj, Instance)
+        self.assertIn("value", obj.instance_attrs)
+        self.assertIn("other", obj.instance_attrs)
+
+    def test_infer_can_assign_has_slots(self) -> None:
+        mod = builder.parse(
+            """
+            class A:
+                __slots__ = ('value',)
+            a = A()
+            a.value = "is set"
+            a.other = "not set"
+        """
+        )
+        obj = list(mod.igetattr("a"))
+        self.assertEqual(len(obj), 1)
+        obj = obj[0]
+        self.assertIsInstance(obj, Instance)
+        self.assertIn("value", obj.instance_attrs)
+        self.assertNotIn("other", obj.instance_attrs)
+
+    def test_infer_can_assign_no_classdict(self) -> None:
+        mod = builder.parse(
+            """
+            a = object()
+            a.value = "not set"
+        """
+        )
+        obj = list(mod.igetattr("a"))
+        self.assertEqual(len(obj), 1)
+        obj = obj[0]
+        self.assertIsInstance(obj, Instance)
+        self.assertNotIn("value", obj.instance_attrs)
+
+    def test_augassign_attr(self) -> None:
         builder.parse(
             """
             class Counter:
@@ -492,7 +546,7 @@ class BuilderTest(unittest.TestCase):
         # TODO: Check self.v += 1 generate AugAssign(AssAttr(...)),
         # not AugAssign(GetAttr(AssName...))
 
-    def test_inferred_dont_pollute(self):
+    def test_inferred_dont_pollute(self) -> None:
         code = """
             def func(a=None):
                 a.custom_attr = 0
@@ -501,14 +555,13 @@ class BuilderTest(unittest.TestCase):
             """
         builder.parse(code)
         nonetype = nodes.const_factory(None)
-        # pylint: disable=no-member; Infers two potential values
         self.assertNotIn("custom_attr", nonetype.locals)
         self.assertNotIn("custom_attr", nonetype.instance_attrs)
         nonetype = nodes.const_factory({})
         self.assertNotIn("custom_attr", nonetype.locals)
         self.assertNotIn("custom_attr", nonetype.instance_attrs)
 
-    def test_asstuple(self):
+    def test_asstuple(self) -> None:
         code = "a, b = range(2)"
         astroid = builder.parse(code)
         self.assertIn("b", astroid.locals)
@@ -519,7 +572,7 @@ class BuilderTest(unittest.TestCase):
         astroid = builder.parse(code)
         self.assertIn("body", astroid["visit_if"].locals)
 
-    def test_build_constants(self):
+    def test_build_constants(self) -> None:
         """test expected values of constants after rebuilding"""
         code = """
             def func():
@@ -528,14 +581,14 @@ class BuilderTest(unittest.TestCase):
                 return 'None'
             """
         astroid = builder.parse(code)
-        none, nothing, chain = [ret.value for ret in astroid.body[0].body]
+        none, nothing, chain = (ret.value for ret in astroid.body[0].body)
         self.assertIsInstance(none, nodes.Const)
         self.assertIsNone(none.value)
         self.assertIsNone(nothing)
         self.assertIsInstance(chain, nodes.Const)
         self.assertEqual(chain.value, "None")
 
-    def test_not_implemented(self):
+    def test_not_implemented(self) -> None:
         node = builder.extract_node(
             """
         NotImplemented #@
@@ -547,10 +600,10 @@ class BuilderTest(unittest.TestCase):
 
 
 class FileBuildTest(unittest.TestCase):
-    def setUp(self):
+    def setUp(self) -> None:
         self.module = resources.build_file("data/module.py", "data.module")
 
-    def test_module_base_props(self):
+    def test_module_base_props(self) -> None:
         """test base properties and method of an astroid module"""
         module = self.module
         self.assertEqual(module.name, "data.module")
@@ -558,15 +611,19 @@ class FileBuildTest(unittest.TestCase):
         self.assertEqual(module.fromlineno, 0)
         self.assertIsNone(module.parent)
         self.assertEqual(module.frame(), module)
+        self.assertEqual(module.frame(future=True), module)
         self.assertEqual(module.root(), module)
         self.assertEqual(module.file, os.path.abspath(resources.find("data/module.py")))
         self.assertEqual(module.pure_python, 1)
         self.assertEqual(module.package, 0)
         self.assertFalse(module.is_statement)
-        self.assertEqual(module.statement(), module)
-        self.assertEqual(module.statement(), module)
+        with pytest.warns(DeprecationWarning) as records:
+            self.assertEqual(module.statement(), module)
+            assert len(records) == 1
+        with self.assertRaises(StatementMissing):
+            module.statement(future=True)
 
-    def test_module_locals(self):
+    def test_module_locals(self) -> None:
         """test the 'locals' dictionary of an astroid module"""
         module = self.module
         _locals = module.locals
@@ -587,7 +644,7 @@ class FileBuildTest(unittest.TestCase):
         should.sort()
         self.assertEqual(keys, sorted(should))
 
-    def test_function_base_props(self):
+    def test_function_base_props(self) -> None:
         """test base properties and method of an astroid function"""
         module = self.module
         function = module["global_access"]
@@ -597,18 +654,20 @@ class FileBuildTest(unittest.TestCase):
         self.assertTrue(function.parent)
         self.assertEqual(function.frame(), function)
         self.assertEqual(function.parent.frame(), module)
+        self.assertEqual(function.frame(future=True), function)
+        self.assertEqual(function.parent.frame(future=True), module)
         self.assertEqual(function.root(), module)
         self.assertEqual([n.name for n in function.args.args], ["key", "val"])
         self.assertEqual(function.type, "function")
 
-    def test_function_locals(self):
+    def test_function_locals(self) -> None:
         """test the 'locals' dictionary of an astroid function"""
         _locals = self.module["global_access"].locals
         self.assertEqual(len(_locals), 4)
         keys = sorted(_locals.keys())
         self.assertEqual(keys, ["i", "key", "local", "val"])
 
-    def test_class_base_props(self):
+    def test_class_base_props(self) -> None:
         """test base properties and method of an astroid class"""
         module = self.module
         klass = module["YO"]
@@ -618,11 +677,13 @@ class FileBuildTest(unittest.TestCase):
         self.assertTrue(klass.parent)
         self.assertEqual(klass.frame(), klass)
         self.assertEqual(klass.parent.frame(), module)
+        self.assertEqual(klass.frame(future=True), klass)
+        self.assertEqual(klass.parent.frame(future=True), module)
         self.assertEqual(klass.root(), module)
         self.assertEqual(klass.basenames, [])
         self.assertTrue(klass.newstyle)
 
-    def test_class_locals(self):
+    def test_class_locals(self) -> None:
         """test the 'locals' dictionary of an astroid class"""
         module = self.module
         klass1 = module["YO"]
@@ -644,21 +705,21 @@ class FileBuildTest(unittest.TestCase):
         ]
         self.assertEqual(sorted(keys), assert_keys)
 
-    def test_class_instance_attrs(self):
+    def test_class_instance_attrs(self) -> None:
         module = self.module
         klass1 = module["YO"]
         klass2 = module["YOUPI"]
         self.assertEqual(list(klass1.instance_attrs.keys()), ["yo"])
         self.assertEqual(list(klass2.instance_attrs.keys()), ["member"])
 
-    def test_class_basenames(self):
+    def test_class_basenames(self) -> None:
         module = self.module
         klass1 = module["YO"]
         klass2 = module["YOUPI"]
         self.assertEqual(klass1.basenames, [])
         self.assertEqual(klass2.basenames, ["YO"])
 
-    def test_method_base_props(self):
+    def test_method_base_props(self) -> None:
         """test base properties and method of an astroid method"""
         klass2 = self.module["YOUPI"]
         # "normal" method
@@ -677,7 +738,7 @@ class FileBuildTest(unittest.TestCase):
         self.assertEqual(method.args.args, [])
         self.assertEqual(method.type, "staticmethod")
 
-    def test_method_locals(self):
+    def test_method_locals(self) -> None:
         """test the 'locals' dictionary of an astroid method"""
         method = self.module["YOUPI"]["method"]
         _locals = method.locals
@@ -686,19 +747,19 @@ class FileBuildTest(unittest.TestCase):
         self.assertEqual(len(_locals), 3)
         self.assertEqual(keys, ["autre", "local", "self"])
 
-    def test_unknown_encoding(self):
-        with self.assertRaises(exceptions.AstroidSyntaxError):
+    def test_unknown_encoding(self) -> None:
+        with self.assertRaises(AstroidSyntaxError):
             resources.build_file("data/invalid_encoding.py")
 
 
-def test_module_build_dunder_file():
+def test_module_build_dunder_file() -> None:
     """Test that module_build() can work with modules that have the *__file__* attribute"""
     module = builder.AstroidBuilder().module_build(collections)
     assert module.path[0] == collections.__file__
 
 
 @pytest.mark.skipif(
-    sys.version_info[:2] >= (3, 8),
+    PY38_PLUS,
     reason=(
         "The builtin ast module does not fail with a specific error "
         "for syntax error caused by invalid type comments."
