@@ -5,29 +5,32 @@
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Keichi Takahashi <keichi.t@me.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
+# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
 # Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
-import builtins
 import unittest
 import xml
 
 import pytest
 
 import astroid
-from astroid import builder, util
-from astroid import exceptions
-from astroid import MANAGER
-from astroid import test_utils
-from astroid import objects
+from astroid import builder, nodes, objects, test_utils, util
+from astroid.exceptions import InferenceError
 
+try:
+    import six  # pylint: disable=unused-import
 
-BUILTINS = MANAGER.astroid_cache[builtins.__name__]
+    HAS_SIX = True
+except ImportError:
+    HAS_SIX = False
 
 
 class InstanceModelTest(unittest.TestCase):
-    def test_instance_special_model(self):
+    def test_instance_special_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A:
@@ -42,7 +45,7 @@ class InstanceModelTest(unittest.TestCase):
         """,
             module_name="fake_module",
         )
-
+        assert isinstance(ast_nodes, list)
         cls = next(ast_nodes[0].infer())
         self.assertIsInstance(cls, astroid.ClassDef)
         self.assertEqual(cls.name, "A")
@@ -79,7 +82,7 @@ class InstanceModelTest(unittest.TestCase):
 
 
 class BoundMethodModelTest(unittest.TestCase):
-    def test_bound_method_model(self):
+    def test_bound_method_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A:
@@ -89,7 +92,7 @@ class BoundMethodModelTest(unittest.TestCase):
         a.test.__self__ #@
         """
         )
-
+        assert isinstance(ast_nodes, list)
         func = next(ast_nodes[0].infer())
         self.assertIsInstance(func, astroid.FunctionDef)
         self.assertEqual(func.name, "test")
@@ -100,7 +103,7 @@ class BoundMethodModelTest(unittest.TestCase):
 
 
 class UnboundMethodModelTest(unittest.TestCase):
-    def test_unbound_method_model(self):
+    def test_unbound_method_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A:
@@ -114,7 +117,7 @@ class UnboundMethodModelTest(unittest.TestCase):
         t.im_self #@
         """
         )
-
+        assert isinstance(ast_nodes, list)
         cls = next(ast_nodes[0].infer())
         self.assertIsInstance(cls, astroid.ClassDef)
         unbound_name = "function"
@@ -135,7 +138,7 @@ class UnboundMethodModelTest(unittest.TestCase):
 
 
 class ClassModelTest(unittest.TestCase):
-    def test_priority_to_local_defined_values(self):
+    def test_priority_to_local_defined_values(self) -> None:
         ast_node = builder.extract_node(
             """
         class A:
@@ -147,7 +150,7 @@ class ClassModelTest(unittest.TestCase):
         self.assertIsInstance(inferred, astroid.Const)
         self.assertEqual(inferred.value, "first")
 
-    def test_class_model_correct_mro_subclasses_proxied(self):
+    def test_class_model_correct_mro_subclasses_proxied(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A(object):
@@ -163,7 +166,7 @@ class ClassModelTest(unittest.TestCase):
             self.assertIsInstance(inferred.bound, astroid.ClassDef)
             self.assertEqual(inferred.bound.name, "type")
 
-    def test_class_model(self):
+    def test_class_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A(object):
@@ -185,7 +188,7 @@ class ClassModelTest(unittest.TestCase):
         """,
             module_name="fake_module",
         )
-
+        assert isinstance(ast_nodes, list)
         module = next(ast_nodes[0].infer())
         self.assertIsInstance(module, astroid.Const)
         self.assertEqual(module.value, "fake_module")
@@ -226,7 +229,7 @@ class ClassModelTest(unittest.TestCase):
 
 
 class ModuleModelTest(unittest.TestCase):
-    def test_priority_to_local_defined_values(self):
+    def test_priority_to_local_defined_values(self) -> None:
         ast_node = astroid.parse(
             """
         __file__ = "mine"
@@ -236,17 +239,17 @@ class ModuleModelTest(unittest.TestCase):
         self.assertIsInstance(file_value, astroid.Const)
         self.assertEqual(file_value.value, "mine")
 
-    def test__path__not_a_package(self):
+    def test__path__not_a_package(self) -> None:
         ast_node = builder.extract_node(
             """
         import sys
         sys.__path__ #@
         """
         )
-        with self.assertRaises(exceptions.InferenceError):
+        with self.assertRaises(InferenceError):
             next(ast_node.infer())
 
-    def test_module_model(self):
+    def test_module_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         import xml
@@ -261,7 +264,7 @@ class ModuleModelTest(unittest.TestCase):
         xml.__dict__ #@
         """
         )
-
+        assert isinstance(ast_nodes, list)
         path = next(ast_nodes[0].infer())
         self.assertIsInstance(path, astroid.List)
         self.assertIsInstance(path.elts[0], astroid.Const)
@@ -292,7 +295,7 @@ class ModuleModelTest(unittest.TestCase):
 
 
 class FunctionModelTest(unittest.TestCase):
-    def test_partial_descriptor_support(self):
+    def test_partial_descriptor_support(self) -> None:
         bound, result = builder.extract_node(
             """
         class A(object): pass
@@ -309,7 +312,7 @@ class FunctionModelTest(unittest.TestCase):
         self.assertIsInstance(result, astroid.Const)
         self.assertEqual(result.value, 42)
 
-    def test___get__has_extra_params_defined(self):
+    def test___get__has_extra_params_defined(self) -> None:
         node = builder.extract_node(
             """
         def test(self): return 42
@@ -350,7 +353,7 @@ class FunctionModelTest(unittest.TestCase):
         self.assertIsInstance(result, astroid.Const)
         self.assertEqual(result.value, 42)
 
-    def test_descriptors_binding_invalid(self):
+    def test_descriptors_binding_invalid(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A: pass
@@ -360,11 +363,10 @@ class FunctionModelTest(unittest.TestCase):
         """
         )
         for node in ast_nodes:
-            with self.assertRaises(exceptions.InferenceError):
+            with self.assertRaises(InferenceError):
                 next(node.infer())
 
-    @pytest.mark.xfail(reason="Relying on path copy")
-    def test_descriptor_error_regression(self):
+    def test_descriptor_error_regression(self) -> None:
         """Make sure the following code does
         node cause an exception"""
         node = builder.extract_node(
@@ -383,10 +385,11 @@ class FunctionModelTest(unittest.TestCase):
         cl #@
         """
         )
+        assert isinstance(node, nodes.NodeNG)
         [const] = node.inferred()
         assert const.value == "MyText"
 
-    def test_function_model(self):
+    def test_function_model(self) -> None:
         ast_nodes = builder.extract_node(
             '''
         def func(a=1, b=2):
@@ -403,7 +406,7 @@ class FunctionModelTest(unittest.TestCase):
         ''',
             module_name="fake_module",
         )
-
+        assert isinstance(ast_nodes, list)
         name = next(ast_nodes[0].infer())
         self.assertIsInstance(name, astroid.Const)
         self.assertEqual(name.value, "func")
@@ -433,7 +436,7 @@ class FunctionModelTest(unittest.TestCase):
         for ast_node in ast_nodes[7:9]:
             self.assertIs(next(ast_node.infer()), astroid.Uninferable)
 
-    def test_empty_return_annotation(self):
+    def test_empty_return_annotation(self) -> None:
         ast_node = builder.extract_node(
             """
         def test(): pass
@@ -444,7 +447,9 @@ class FunctionModelTest(unittest.TestCase):
         self.assertIsInstance(annotations, astroid.Dict)
         self.assertEqual(len(annotations.items), 0)
 
-    def test_builtin_dunder_init_does_not_crash_when_accessing_annotations(self):
+    def test_builtin_dunder_init_does_not_crash_when_accessing_annotations(
+        self,
+    ) -> None:
         ast_node = builder.extract_node(
             """
         class Class:
@@ -457,7 +462,7 @@ class FunctionModelTest(unittest.TestCase):
         self.assertIsInstance(inferred, astroid.Dict)
         self.assertEqual(len(inferred.items), 0)
 
-    def test_annotations_kwdefaults(self):
+    def test_annotations_kwdefaults(self) -> None:
         ast_node = builder.extract_node(
             """
         def test(a: 1, *args: 2, f:4='lala', **kwarg:3)->2: pass
@@ -500,7 +505,7 @@ class FunctionModelTest(unittest.TestCase):
 
 
 class GeneratorModelTest(unittest.TestCase):
-    def test_model(self):
+    def test_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         def test():
@@ -515,7 +520,7 @@ class GeneratorModelTest(unittest.TestCase):
         gen.send #@
         """
         )
-
+        assert isinstance(ast_nodes, list)
         name = next(ast_nodes[0].infer())
         self.assertEqual(name.value, "test")
 
@@ -535,7 +540,7 @@ class GeneratorModelTest(unittest.TestCase):
 
 
 class ExceptionModelTest(unittest.TestCase):
-    def test_valueerror_py3(self):
+    def test_valueerror_py3(self) -> None:
         ast_nodes = builder.extract_node(
             """
         try:
@@ -547,16 +552,17 @@ class ExceptionModelTest(unittest.TestCase):
            err.message #@
         """
         )
+        assert isinstance(ast_nodes, list)
         args = next(ast_nodes[0].infer())
         self.assertIsInstance(args, astroid.Tuple)
         tb = next(ast_nodes[1].infer())
         self.assertIsInstance(tb, astroid.Instance)
         self.assertEqual(tb.name, "traceback")
 
-        with self.assertRaises(exceptions.InferenceError):
+        with self.assertRaises(InferenceError):
             next(ast_nodes[2].infer())
 
-    def test_syntax_error(self):
+    def test_syntax_error(self) -> None:
         ast_node = builder.extract_node(
             """
         try:
@@ -568,7 +574,8 @@ class ExceptionModelTest(unittest.TestCase):
         inferred = next(ast_node.infer())
         assert isinstance(inferred, astroid.Const)
 
-    def test_oserror(self):
+    @unittest.skipIf(HAS_SIX, "This test fails if the six library is installed")
+    def test_oserror(self) -> None:
         ast_nodes = builder.extract_node(
             """
         try:
@@ -585,7 +592,7 @@ class ExceptionModelTest(unittest.TestCase):
             assert isinstance(inferred, astroid.Const)
             assert inferred.value == value
 
-    def test_unicodedecodeerror(self):
+    def test_unicodedecodeerror(self) -> None:
         code = """
         try:
             raise UnicodeDecodeError("utf-8", "blob", 0, 1, "reason")
@@ -596,7 +603,7 @@ class ExceptionModelTest(unittest.TestCase):
         inferred = next(node.infer())
         assert isinstance(inferred, astroid.Const)
 
-    def test_import_error(self):
+    def test_import_error(self) -> None:
         ast_nodes = builder.extract_node(
             """
         try:
@@ -611,7 +618,7 @@ class ExceptionModelTest(unittest.TestCase):
             assert isinstance(inferred, astroid.Const)
             assert inferred.value == ""
 
-    def test_exception_instance_correctly_instantiated(self):
+    def test_exception_instance_correctly_instantiated(self) -> None:
         ast_node = builder.extract_node(
             """
         try:
@@ -627,13 +634,13 @@ class ExceptionModelTest(unittest.TestCase):
 
 
 class DictObjectModelTest(unittest.TestCase):
-    def test__class__(self):
+    def test__class__(self) -> None:
         ast_node = builder.extract_node("{}.__class__")
         inferred = next(ast_node.infer())
         self.assertIsInstance(inferred, astroid.ClassDef)
         self.assertEqual(inferred.name, "dict")
 
-    def test_attributes_inferred_as_methods(self):
+    def test_attributes_inferred_as_methods(self) -> None:
         ast_nodes = builder.extract_node(
             """
         {}.values #@
@@ -645,7 +652,7 @@ class DictObjectModelTest(unittest.TestCase):
             inferred = next(node.infer())
             self.assertIsInstance(inferred, astroid.BoundMethod)
 
-    def test_wrapper_objects_for_dict_methods_python3(self):
+    def test_wrapper_objects_for_dict_methods_python3(self) -> None:
         ast_nodes = builder.extract_node(
             """
         {1:1, 2:3}.values() #@
@@ -653,6 +660,7 @@ class DictObjectModelTest(unittest.TestCase):
         {1:1, 2:3}.items() #@
         """
         )
+        assert isinstance(ast_nodes, list)
         values = next(ast_nodes[0].infer())
         self.assertIsInstance(values, objects.DictValues)
         self.assertEqual([elt.value for elt in values.elts], [1, 3])
@@ -664,7 +672,7 @@ class DictObjectModelTest(unittest.TestCase):
 
 
 class LruCacheModelTest(unittest.TestCase):
-    def test_lru_cache(self):
+    def test_lru_cache(self) -> None:
         ast_nodes = builder.extract_node(
             """
         import functools
@@ -678,6 +686,7 @@ class LruCacheModelTest(unittest.TestCase):
         f.foo.cache_info() #@
         """
         )
+        assert isinstance(ast_nodes, list)
         cache_clear = next(ast_nodes[0].infer())
         self.assertIsInstance(cache_clear, astroid.BoundMethod)
         wrapped = next(ast_nodes[1].infer())
