@@ -3,23 +3,24 @@
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Copyright (c) 2021 hippo91 <guillaume.peillex@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 
 import unittest
+from typing import List
 
-from astroid import bases
-from astroid import builder
-from astroid import exceptions
-from astroid import nodes
-from astroid import objects
+from astroid import bases, builder, nodes, objects
+from astroid.exceptions import AttributeInferenceError, InferenceError, SuperError
+from astroid.objects import Super
 
 
 class ObjectsTest(unittest.TestCase):
-    def test_frozenset(self):
+    def test_frozenset(self) -> None:
         node = builder.extract_node(
             """
         frozenset({1: 2, 2: 3}) #@
@@ -28,7 +29,7 @@ class ObjectsTest(unittest.TestCase):
         inferred = next(node.infer())
         self.assertIsInstance(inferred, objects.FrozenSet)
 
-        self.assertEqual(inferred.pytype(), "%s.frozenset" % bases.BUILTINS)
+        self.assertEqual(inferred.pytype(), "builtins.frozenset")
 
         itered = inferred.itered()
         self.assertEqual(len(itered), 2)
@@ -36,12 +37,12 @@ class ObjectsTest(unittest.TestCase):
         self.assertEqual([const.value for const in itered], [1, 2])
 
         proxied = inferred._proxied
-        self.assertEqual(inferred.qname(), "%s.frozenset" % bases.BUILTINS)
+        self.assertEqual(inferred.qname(), "builtins.frozenset")
         self.assertIsInstance(proxied, nodes.ClassDef)
 
 
 class SuperTests(unittest.TestCase):
-    def test_inferring_super_outside_methods(self):
+    def test_inferring_super_outside_methods(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class Module(object):
@@ -57,19 +58,20 @@ class SuperTests(unittest.TestCase):
         super() #@
         """
         )
+        assert isinstance(ast_nodes, list)
         in_static = next(ast_nodes[0].value.infer())
         self.assertIsInstance(in_static, bases.Instance)
-        self.assertEqual(in_static.qname(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(in_static.qname(), "builtins.super")
 
         module_level = next(ast_nodes[1].infer())
         self.assertIsInstance(module_level, bases.Instance)
-        self.assertEqual(in_static.qname(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(in_static.qname(), "builtins.super")
 
         no_arguments = next(ast_nodes[2].infer())
         self.assertIsInstance(no_arguments, bases.Instance)
-        self.assertEqual(no_arguments.qname(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(no_arguments.qname(), "builtins.super")
 
-    def test_inferring_unbound_super_doesnt_work(self):
+    def test_inferring_unbound_super_doesnt_work(self) -> None:
         node = builder.extract_node(
             """
         class Test(object):
@@ -79,9 +81,9 @@ class SuperTests(unittest.TestCase):
         )
         unbounded = next(node.infer())
         self.assertIsInstance(unbounded, bases.Instance)
-        self.assertEqual(unbounded.qname(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(unbounded.qname(), "builtins.super")
 
-    def test_use_default_inference_on_not_inferring_args(self):
+    def test_use_default_inference_on_not_inferring_args(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class Test(object):
@@ -90,15 +92,16 @@ class SuperTests(unittest.TestCase):
                 super(Test, lala) #@
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, bases.Instance)
-        self.assertEqual(first.qname(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(first.qname(), "builtins.super")
 
         second = next(ast_nodes[1].infer())
         self.assertIsInstance(second, bases.Instance)
-        self.assertEqual(second.qname(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(second.qname(), "builtins.super")
 
-    def test_no_arguments_super(self):
+    def test_no_arguments_super(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class First(object): pass
@@ -110,6 +113,7 @@ class SuperTests(unittest.TestCase):
                 super() #@
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, objects.Super)
         self.assertIsInstance(first.type, bases.Instance)
@@ -124,7 +128,7 @@ class SuperTests(unittest.TestCase):
         self.assertIsInstance(second.mro_pointer, nodes.ClassDef)
         self.assertEqual(second.mro_pointer.name, "Second")
 
-    def test_super_simple_cases(self):
+    def test_super_simple_cases(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class First(object): pass
@@ -149,6 +153,7 @@ class SuperTests(unittest.TestCase):
         # the lookup should be done.
 
         # super(Third, self)
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, objects.Super)
         self.assertIsInstance(first.type, bases.Instance)
@@ -188,7 +193,7 @@ class SuperTests(unittest.TestCase):
         self.assertIsInstance(fifth.mro_pointer, nodes.ClassDef)
         self.assertEqual(fifth.mro_pointer.name, "Fourth")
 
-    def test_super_infer(self):
+    def test_super_infer(self) -> None:
         node = builder.extract_node(
             """
         class Super(object):
@@ -202,7 +207,7 @@ class SuperTests(unittest.TestCase):
         self.assertIsInstance(reinferred, objects.Super)
         self.assertIs(inferred, reinferred)
 
-    def test_inferring_invalid_supers(self):
+    def test_inferring_invalid_supers(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class Super(object):
@@ -217,20 +222,21 @@ class SuperTests(unittest.TestCase):
             pass
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, objects.Super)
-        with self.assertRaises(exceptions.SuperError) as cm:
+        with self.assertRaises(SuperError) as cm:
             first.super_mro()
         self.assertIsInstance(cm.exception.super_.mro_pointer, nodes.Const)
         self.assertEqual(cm.exception.super_.mro_pointer.value, 1)
         for node, invalid_type in zip(ast_nodes[1:], (nodes.Const, bases.Instance)):
             inferred = next(node.infer())
             self.assertIsInstance(inferred, objects.Super, node)
-            with self.assertRaises(exceptions.SuperError) as cm:
+            with self.assertRaises(SuperError) as cm:
                 inferred.super_mro()
             self.assertIsInstance(cm.exception.super_.type, invalid_type)
 
-    def test_proxied(self):
+    def test_proxied(self) -> None:
         node = builder.extract_node(
             """
         class Super(object):
@@ -240,10 +246,10 @@ class SuperTests(unittest.TestCase):
         )
         inferred = next(node.infer())
         proxied = inferred._proxied
-        self.assertEqual(proxied.qname(), "%s.super" % bases.BUILTINS)
+        self.assertEqual(proxied.qname(), "builtins.super")
         self.assertIsInstance(proxied, nodes.ClassDef)
 
-    def test_super_bound_model(self):
+    def test_super_bound_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class First(object):
@@ -268,6 +274,7 @@ class SuperTests(unittest.TestCase):
         """
         )
         # Super(type, type) is the same for both functions and classmethods.
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, nodes.FunctionDef)
         self.assertEqual(first.name, "method")
@@ -298,7 +305,7 @@ class SuperTests(unittest.TestCase):
         self.assertEqual(sixth.bound.name, "First")
         self.assertEqual(sixth.type, "classmethod")
 
-    def test_super_getattr_single_inheritance(self):
+    def test_super_getattr_single_inheritance(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class First(object):
@@ -320,6 +327,7 @@ class SuperTests(unittest.TestCase):
 
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, bases.BoundMethod)
         self.assertEqual(first.bound.name, "Second")
@@ -328,12 +336,12 @@ class SuperTests(unittest.TestCase):
         self.assertIsInstance(second, bases.BoundMethod)
         self.assertEqual(second.bound.name, "First")
 
-        with self.assertRaises(exceptions.InferenceError):
+        with self.assertRaises(InferenceError):
             next(ast_nodes[2].infer())
         fourth = next(ast_nodes[3].infer())
-        with self.assertRaises(exceptions.AttributeInferenceError):
+        with self.assertRaises(AttributeInferenceError):
             fourth.getattr("test3")
-        with self.assertRaises(exceptions.AttributeInferenceError):
+        with self.assertRaises(AttributeInferenceError):
             next(fourth.igetattr("test3"))
 
         first_unbound = next(ast_nodes[4].infer())
@@ -346,7 +354,7 @@ class SuperTests(unittest.TestCase):
         self.assertEqual(second_unbound.name, "test")
         self.assertEqual(second_unbound.parent.name, "First")
 
-    def test_super_invalid_mro(self):
+    def test_super_invalid_mro(self) -> None:
         node = builder.extract_node(
             """
         class A(object):
@@ -357,10 +365,10 @@ class SuperTests(unittest.TestCase):
         """
         )
         inferred = next(node.infer())
-        with self.assertRaises(exceptions.AttributeInferenceError):
+        with self.assertRaises(AttributeInferenceError):
             next(inferred.getattr("test"))
 
-    def test_super_complex_mro(self):
+    def test_super_complex_mro(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A(object):
@@ -382,6 +390,7 @@ class SuperTests(unittest.TestCase):
                 super(E, self).static #@
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, bases.BoundMethod)
         self.assertEqual(first.bound.name, "C")
@@ -397,7 +406,7 @@ class SuperTests(unittest.TestCase):
         self.assertIsInstance(static, nodes.FunctionDef)
         self.assertEqual(static.parent.scope().name, "A")
 
-    def test_super_data_model(self):
+    def test_super_data_model(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class X(object): pass
@@ -408,6 +417,7 @@ class SuperTests(unittest.TestCase):
                 super(X, A) #@
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         thisclass = first.getattr("__thisclass__")[0]
         self.assertIsInstance(thisclass, nodes.ClassDef)
@@ -434,10 +444,10 @@ class SuperTests(unittest.TestCase):
         selfclass = third.getattr("__self_class__")[0]
         self.assertEqual(selfclass.name, "A")
 
-    def assertEqualMro(self, klass, expected_mro):
+    def assertEqualMro(self, klass: Super, expected_mro: List[str]) -> None:
         self.assertEqual([member.name for member in klass.super_mro()], expected_mro)
 
-    def test_super_mro(self):
+    def test_super_mro(self) -> None:
         ast_nodes = builder.extract_node(
             """
         class A(object): pass
@@ -453,6 +463,7 @@ class SuperTests(unittest.TestCase):
                 super(1, B) #@
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertEqualMro(first, ["C", "B", "A", "object"])
         second = next(ast_nodes[1].infer())
@@ -461,13 +472,13 @@ class SuperTests(unittest.TestCase):
         self.assertEqualMro(third, ["A", "object"])
 
         fourth = next(ast_nodes[3].infer())
-        with self.assertRaises(exceptions.SuperError):
+        with self.assertRaises(SuperError):
             fourth.super_mro()
         fifth = next(ast_nodes[4].infer())
-        with self.assertRaises(exceptions.SuperError):
+        with self.assertRaises(SuperError):
             fifth.super_mro()
 
-    def test_super_yes_objects(self):
+    def test_super_yes_objects(self) -> None:
         ast_nodes = builder.extract_node(
             """
         from collections import Missing
@@ -477,12 +488,13 @@ class SuperTests(unittest.TestCase):
                 super(A, Missing) #@
         """
         )
+        assert isinstance(ast_nodes, list)
         first = next(ast_nodes[0].infer())
         self.assertIsInstance(first, bases.Instance)
         second = next(ast_nodes[1].infer())
         self.assertIsInstance(second, bases.Instance)
 
-    def test_super_invalid_types(self):
+    def test_super_invalid_types(self) -> None:
         node = builder.extract_node(
             """
         import collections
@@ -492,12 +504,12 @@ class SuperTests(unittest.TestCase):
         """
         )
         inferred = next(node.infer())
-        with self.assertRaises(exceptions.SuperError):
+        with self.assertRaises(SuperError):
             inferred.super_mro()
-        with self.assertRaises(exceptions.SuperError):
+        with self.assertRaises(SuperError):
             inferred.super_mro()
 
-    def test_super_properties(self):
+    def test_super_properties(self) -> None:
         node = builder.extract_node(
             """
         class Foo(object):
@@ -517,7 +529,7 @@ class SuperTests(unittest.TestCase):
         self.assertIsInstance(inferred, nodes.Const)
         self.assertEqual(inferred.value, 42)
 
-    def test_super_qname(self):
+    def test_super_qname(self) -> None:
         """Make sure a Super object generates a qname
         equivalent to super.__qname__
         """
