@@ -7,64 +7,73 @@
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
+# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 import platform
 import unittest
 
 import _io
+
 from astroid.builder import AstroidBuilder
 from astroid.raw_building import (
     attach_dummy_node,
-    build_module,
     build_class,
-    build_function,
     build_from_import,
+    build_function,
+    build_module,
 )
 
 
 class RawBuildingTC(unittest.TestCase):
-    def test_attach_dummy_node(self):
+    def test_attach_dummy_node(self) -> None:
         node = build_module("MyModule")
         attach_dummy_node(node, "DummyNode")
         self.assertEqual(1, len(list(node.get_children())))
 
-    def test_build_module(self):
+    def test_build_module(self) -> None:
         node = build_module("MyModule")
         self.assertEqual(node.name, "MyModule")
         self.assertEqual(node.pure_python, False)
         self.assertEqual(node.package, False)
         self.assertEqual(node.parent, None)
 
-    def test_build_class(self):
+    def test_build_class(self) -> None:
         node = build_class("MyClass")
         self.assertEqual(node.name, "MyClass")
         self.assertEqual(node.doc, None)
 
-    def test_build_function(self):
+    def test_build_function(self) -> None:
         node = build_function("MyFunction")
         self.assertEqual(node.name, "MyFunction")
         self.assertEqual(node.doc, None)
 
-    def test_build_function_args(self):
+    def test_build_function_args(self) -> None:
         args = ["myArgs1", "myArgs2"]
         node = build_function("MyFunction", args)
         self.assertEqual("myArgs1", node.args.args[0].name)
         self.assertEqual("myArgs2", node.args.args[1].name)
         self.assertEqual(2, len(node.args.args))
 
-    def test_build_function_defaults(self):
+    def test_build_function_defaults(self) -> None:
         defaults = ["defaults1", "defaults2"]
         node = build_function(name="MyFunction", args=None, defaults=defaults)
         self.assertEqual(2, len(node.args.defaults))
 
-    def test_build_function_posonlyargs(self):
+    def test_build_function_posonlyargs(self) -> None:
         node = build_function(name="MyFunction", posonlyargs=["a", "b"])
         self.assertEqual(2, len(node.args.posonlyargs))
 
-    def test_build_from_import(self):
+    def test_build_function_kwonlyargs(self) -> None:
+        node = build_function(name="MyFunction", kwonlyargs=["a", "b"])
+        assert len(node.args.kwonlyargs) == 2
+        assert node.args.kwonlyargs[0].name == "a"
+        assert node.args.kwonlyargs[1].name == "b"
+
+    def test_build_from_import(self) -> None:
         names = ["exceptions, inference, inspector"]
         node = build_from_import("astroid", names)
         self.assertEqual(len(names), len(node.names))
